@@ -788,7 +788,7 @@ xmlParseCatalogFile(const char *filename) {
 	return(NULL);
     }
 
-    inputStream->filename = xmlCanonicPath(filename);
+    inputStream->filename = (char *) xmlCanonicPath((const xmlChar *)filename);
     inputStream->buf = buf;
     inputStream->base = inputStream->buf->buffer->content;
     inputStream->cur = inputStream->buf->buffer->content;
@@ -2909,8 +2909,10 @@ xmlLoadCatalog(const char *filename)
 
     if (xmlDefaultCatalog == NULL) {
 	catal = xmlLoadACatalog(filename);
-	if (catal == NULL)
+	if (catal == NULL) {
+	    xmlRMutexUnlock(xmlCatalogMutex);
 	    return(-1);
+	}
 
 	xmlDefaultCatalog = catal;
 	xmlRMutexUnlock(xmlCatalogMutex);
