@@ -39,8 +39,8 @@
 #include <errno.h>
 #endif
 #endif
-#include <libxml/xmlmemory.h>
 #include <libxml/encoding.h>
+#include <libxml/xmlmemory.h>
 #ifdef LIBXML_HTML_ENABLED
 #include <libxml/HTMLparser.h>
 #endif
@@ -135,7 +135,7 @@ xmlUTF8Strlen(const xmlChar *utf) {
  * Returns the char value or -1 in case of error and update @len with the
  *        number of bytes used
  */
-static int
+int
 xmlGetUTF8Char(const unsigned char *utf, int *len) {
     unsigned int c;
 
@@ -193,7 +193,7 @@ error:
 }
 
 /**
- * xmlCheckUTF8: Check utf-8 string for legality.
+ * xmlCheckUTF8:
  * @utf: Pointer to putative utf-8 encoded string.
  *
  * Checks @utf for being valid utf-8. @utf is assumed to be
@@ -283,7 +283,7 @@ xmlUTF8Strndup(const xmlChar *utf, int len) {
     
     if ((utf == NULL) || (len < 0)) return(NULL);
     i = xmlUTF8Strsize(utf, len);
-    ret = (xmlChar *) xmlMalloc((i + 1) * sizeof(xmlChar));
+    ret = (xmlChar *) xmlMallocAtomic((i + 1) * sizeof(xmlChar));
     if (ret == NULL) {
         xmlGenericError(xmlGenericErrorContext,
 		"malloc of %ld byte failed\n",
@@ -591,6 +591,43 @@ isolat1ToUTF8(unsigned char* out, int *outlen,
     return(0);
 }
 
+/**
+ * UTF8ToUTF8:
+ * @out:  a pointer to an array of bytes to store the result
+ * @outlen:  the length of @out
+ * @inb:  a pointer to an array of UTF-8 chars
+ * @inlenb:  the length of @in in UTF-8 chars
+ *
+ * No op copy operation for UTF8 handling.
+ *
+ * Returns the number of byte written, or -1 by lack of space, or -2
+ *     if the transcoding fails (for *in is not valid utf16 string)
+ *     The value of *inlen after return is the number of octets consumed
+ *     as the return value is positive, else unpredictable.
+ */
+static int
+UTF8ToUTF8(unsigned char* out, int *outlen,
+           const unsigned char* inb, int *inlenb)
+{
+    int len;
+
+    if ((out == NULL) || (inb == NULL) || (outlen == NULL) || (inlenb == NULL))
+	return(-1);
+    if (*outlen > *inlenb) {
+	len = *inlenb;
+    } else {
+	len = *outlen;
+    }
+    if (len < 0)
+	return(-1);
+
+    memcpy(out, inb, len);
+
+    *outlen = len;
+    *inlenb = len;
+    return(0);
+}
+
 
 /**
  * UTF8Toisolat1:
@@ -1481,9 +1518,10 @@ static xmlCharEncodingHandlerPtr xmlDefaultCharEncodingHandler = NULL;
  * @output:  the xmlCharEncodingOutputFunc to write that encoding
  *
  * Create and registers an xmlCharEncodingHandler.
+ *
  * Returns the xmlCharEncodingHandlerPtr created (or NULL in case of error).
  */
-static xmlCharEncodingHandlerPtr
+xmlCharEncodingHandlerPtr
 xmlNewCharEncodingHandler(const char *name, 
                           xmlCharEncodingInputFunc input,
                           xmlCharEncodingOutputFunc output) {
@@ -1578,7 +1616,7 @@ xmlInitCharEncodingHandlers(void) {
 		"xmlInitCharEncodingHandlers : out of memory !\n");
 	return;
     }
-    xmlNewCharEncodingHandler("UTF-8", NULL, NULL);
+    xmlNewCharEncodingHandler("UTF-8", UTF8ToUTF8, UTF8ToUTF8);
     xmlUTF16LEHandler = 
           xmlNewCharEncodingHandler("UTF-16LE", UTF16LEToUTF8, UTF8ToUTF16LE);
     xmlUTF16BEHandler = 
@@ -2119,7 +2157,7 @@ xmlCharEncInFunc(xmlCharEncodingHandler * handler, xmlBufferPtr out,
      */
     if (ret == -3)
         ret = 0;
-    return (ret);
+    return (written);
 }
 
 /**
@@ -2156,6 +2194,9 @@ retry:
     
     written = out->size - out->use;
 
+    if (written > 0)
+	written--; /* Gennady: count '/0' */
+
     /*
      * First specific handling of in = NULL, i.e. the initialization call
      */
@@ -2164,8 +2205,10 @@ retry:
 	if (handler->output != NULL) {
 	    ret = handler->output(&out->content[out->use], &written,
 				  NULL, &toconv);
-	    out->use += written;
-	    out->content[out->use] = 0;
+	    if (ret >= 0) { /* Gennady: check return value */
+		out->use += written;
+		out->content[out->use] = 0;
+	    }
 	}
 #ifdef LIBXML_ICONV_ENABLED
 	else if (handler->iconv_out != NULL) {
@@ -2245,8 +2288,10 @@ retry:
 #endif
 	    break;
         case -3:
+#ifdef DEBUG_ENCODING
 	    xmlGenericError(xmlGenericErrorContext,"converted %d bytes to %d bytes of output %d left\n",
 	            toconv, written, in->use);
+#endif
 	    break;
         case -2: {
 	    int len = in->use;
