@@ -865,8 +865,6 @@ def buildWrappers():
 		func = nameFixup(name, classe, type, file)
 		info = (1, func, name, ret, args, file)
 		function_classes[classe].append(info)
-	    if found == 1:
-		break
 	if found == 1:
 	    continue
 	if name[0:8] == "xmlXPath":
