@@ -92,7 +92,7 @@ static pthread_once_t once_control = PTHREAD_ONCE_INIT;
 static __declspec(thread) xmlGlobalState tlstate;
 static __declspec(thread) int tlstate_inited = 0;
 #else /* HAVE_COMPILER_TLS */
-static DWORD globalkey;
+static DWORD globalkey = TLS_OUT_OF_INDEXES;
 #endif /* HAVE_COMPILER_TLS */
 static DWORD mainthread;
 static int run_once_init = 1;
@@ -102,7 +102,7 @@ static xmlRMutexPtr	xmlLibraryLock = NULL;
 static void xmlOnceInit(void);
 
 /**
- * xmlMutexPtr:
+ * xmlNewMutex:
  *
  * xmlNewMutex() is used to allocate a libxml2 token struct for use in
  * synchronizing access to data.
@@ -176,7 +176,7 @@ xmlMutexUnlock(xmlMutexPtr tok ATTRIBUTE_UNUSED)
 }
 
 /**
- * xmlRNewMutex:
+ * xmlNewRMutex:
  *
  * xmlRNewMutex() is used to allocate a reentrant mutex for use in
  * synchronizing access to data. token_r is a re-entrant lock and thus useful
@@ -196,6 +196,7 @@ xmlNewRMutex(void)
     pthread_mutex_init(&tok->lock, NULL);
     tok->held = 0;
     tok->waiters = 0;
+    pthread_cond_init(&tok->cv, NULL);
 #elif defined HAVE_WIN32_THREADS
     InitializeCriticalSection(&tok->cs);
     tok->count = 0;
@@ -204,7 +205,7 @@ xmlNewRMutex(void)
 }
 
 /**
- * xmlRFreeMutex:
+ * xmlFreeRMutex:
  * @tok:  the reentrant mutex
  *
  * xmlRFreeMutex() is used to reclaim resources associated with a
@@ -322,23 +323,15 @@ xmlNewGlobalState(void)
 #endif /* LIBXML_THREAD_ENABLED */
 
 
-/**
- * xmlGetGlobalState:
- *
- * xmlGetGlobalState() is called to retrieve the global state for a thread.
- *
- * Returns the thread global state or NULL in case of error
- */
-
 #ifdef HAVE_WIN32_THREADS
-#if !defined(HAVE_COMPILER_TLS)
+#if !defined(HAVE_COMPILER_TLS) && defined(LIBXML_STATIC)
 typedef struct _xmlGlobalStateCleanupHelperParams
 {
     HANDLE thread;
     void *memory;
 } xmlGlobalStateCleanupHelperParams;
 
-void xmlGlobalStateCleanupHelper (void *p)
+static void xmlGlobalStateCleanupHelper (void *p)
 {
     xmlGlobalStateCleanupHelperParams *params = (xmlGlobalStateCleanupHelperParams *) p;
     WaitForSingleObject(params->thread, INFINITE);
@@ -347,9 +340,16 @@ void xmlGlobalStateCleanupHelper (void *p)
     free(params);
     _endthread();
 }
-#endif /* HAVE_COMPILER_TLS */
+#endif /* HAVE_COMPILER_TLS && LIBXML_STATIC */
 #endif /* HAVE_WIN32_THREADS */
 
+/**
+ * xmlGetGlobalState:
+ *
+ * xmlGetGlobalState() is called to retrieve the global state for a thread.
+ *
+ * Returns the thread global state or NULL in case of error
+ */
 xmlGlobalStatePtr
 xmlGetGlobalState(void)
 {
@@ -382,13 +382,17 @@ xmlGetGlobalState(void)
     }
     if ((globalval = (xmlGlobalState *) TlsGetValue(globalkey)) == NULL) {
 	xmlGlobalState *tsd = xmlNewGlobalState();
+#if defined(LIBXML_STATIC)
 	xmlGlobalStateCleanupHelperParams *p = 
 	    (xmlGlobalStateCleanupHelperParams *) malloc(sizeof(xmlGlobalStateCleanupHelperParams));
 	p->memory = tsd;
 	DuplicateHandle(GetCurrentProcess(), GetCurrentThread(), 
 		GetCurrentProcess(), &p->thread, 0, TRUE, DUPLICATE_SAME_ACCESS);
+#endif
 	TlsSetValue(globalkey, tsd);
+#if defined(LIBXML_STATIC)
 	_beginthread(xmlGlobalStateCleanupHelper, 0, p);
+#endif
 
 	return (tsd);
     }
@@ -536,3 +540,34 @@ xmlOnceInit(void) {
     mainthread = GetCurrentThreadId();
 #endif
 }
+
+/**
+ * DllMain
+ *
+ * Entry point for Windows library. It is being used to free thread-specific
+ * storage.
+ */
+#if defined(HAVE_WIN32_THREADS) && !defined(LIBXML_STATIC)
+BOOL WINAPI DllMain(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved) 
+{
+    switch(fdwReason) {
+    case DLL_THREAD_DETACH:
+	if (globalkey != TLS_OUT_OF_INDEXES) {
+	    xmlGlobalState *globalval = (xmlGlobalState *)TlsGetValue(globalkey);
+	    if (globalval) {
+		xmlFreeGlobalState(globalval);
+		TlsSetValue(globalkey, NULL);
+	    }
+	}
+	break;
+    case DLL_PROCESS_DETACH:
+	if (globalkey != TLS_OUT_OF_INDEXES) {
+	    TlsFree(globalkey);
+	    globalkey = TLS_OUT_OF_INDEXES;
+	}
+	break;
+    }
+    return TRUE;
+}
+#endif
+
