@@ -688,6 +688,8 @@ xmlTextReaderDoExpand(xmlTextReaderPtr reader) {
     do {
         if (xmlTextReaderGetSuccessor(reader->node) != NULL)
 	    return(1);
+	if (reader->ctxt->nodeNr <= reader->depth)
+	    return(1);
 	if (reader->mode == XML_TEXTREADER_MODE_EOF)
 	    return(1);
 	val = xmlTextReaderPushData(reader);
@@ -940,7 +942,7 @@ xmlTextReaderReadState(xmlTextReaderPtr reader) {
  * @reader:  the xmlTextReaderPtr used
  *
  * Reads the contents of the current node and the full subtree. It then makes
- * the subtree availsble until the next xmlTextReaderRead() call
+ * the subtree available until the next xmlTextReaderRead() call
  *
  * Returns a node pointer valid until the next xmlTextReaderRead() call
  *         or NULL in case of error.
@@ -1208,6 +1210,7 @@ void
 xmlFreeTextReader(xmlTextReaderPtr reader) {
     if (reader == NULL)
 	return;
+#ifdef LIBXML_SCHEMAS_ENABLED
     if (reader->rngSchemas != NULL) {
 	xmlRelaxNGFree(reader->rngSchemas);
 	reader->rngSchemas = NULL;
@@ -1216,6 +1219,7 @@ xmlFreeTextReader(xmlTextReaderPtr reader) {
 	xmlRelaxNGFreeValidCtxt(reader->rngValidCtxt);
 	reader->rngValidCtxt = NULL;
     }
+#endif
     if (reader->ctxt != NULL) {
 	if (reader->ctxt->myDoc != NULL) {
 	    xmlFreeDoc(reader->ctxt->myDoc);
@@ -2473,6 +2477,7 @@ xmlTextReaderCurrentDoc(xmlTextReaderPtr reader) {
     return(reader->ctxt->myDoc);
 }
 
+#ifdef LIBXML_SCHEMAS_ENABLED
 /**
  * xmlTextReaderRelaxNGSetSchema:
  * @reader:  the xmlTextReaderPtr used
@@ -2590,6 +2595,7 @@ xmlTextReaderRelaxNGValidate(xmlTextReaderPtr reader, const char *rng) {
     reader->validate = XML_TEXTREADER_VALIDATE_RNG;
     return(0);
 }
+#endif
 
 /************************************************************************
  *									*
